pub fn process_part1(input: &str) -> i32 {
    let result = input
        .lines()
        .map(|round| {
            let (opponent_move, my_move) = round.split_once(" ").unwrap();
            get_points(opponent_move, my_move)
        })
        .sum();
    result
}

fn get_points(opponent_move: &str, my_move: &str) -> i32 {
    let mut round_score = 0;
    match my_move {
        "X" => {
            round_score += 1;
            match opponent_move {
                "C" => round_score += 6,
                "A" => round_score += 3,
                _ => (),
            }
        }
        "Y" => {
            round_score += 2;
            match opponent_move {
                "A" => round_score += 6,
                "B" => round_score += 3,
                _ => (),
            }
        }
        "Z" => {
            round_score += 3;
            match opponent_move {
                "B" => round_score += 6,
                "C" => round_score += 3,
                _ => (),
            }
        }
        _ => panic!("Shouldn't happen"),
    }
    round_score
}

pub fn process_part2(input: &str) -> i32 {
    let result = input
        .lines()
        .map(|round| {
            let (opponent_move, round_result) = round.split_once(" ").unwrap();
            get_points_fixed(opponent_move, round_result)
        })
        .sum();
    result
}

fn get_points_fixed(opponent_move: &str, round_result: &str) -> i32 {
    let mut round_score = 0;
    match round_result {
        "X" => match opponent_move {
            "A" => round_score += 3,
            "B" => round_score += 1,
            "C" => round_score += 2,
            _ => (),
        },
        "Y" => {
            round_score += 3;
            match opponent_move {
                "A" => round_score += 1,
                "B" => round_score += 2,
                "C" => round_score += 3,
                _ => (),
            }
        }
        "Z" => {
            round_score += 6;
            match opponent_move {
                "A" => round_score += 2,
                "B" => round_score += 3,
                "C" => round_score += 1,
                _ => (),
            }
        }
        _ => panic!("Shouldn't happen"),
    }
    round_score
}

#[cfg(test)]
mod tests {
    use super::*;

    const INPUT: &str = "A Y
B X
C Z";

    #[test]
    fn part1() {
        let result = process_part1(INPUT);
        assert_eq!(result, 15);
    }

    #[test]
    fn part2() {
        let result = process_part2(INPUT);
        assert_eq!(result, 12);
    }
}
