fn char_to_prio(char: char) -> i32 {
    let ascii = char as i32;
    match ascii {
        65..=90 => ascii - 38,
        97..=122 => ascii - 96,
        _ => panic!("Shouldn't happen"),
    }
}

pub fn process_part1(input: &str) -> i32 {
    input
        .lines()
        .map(|line| {
            let (first, second) = line.split_at(line.len() / 2);
            let mut prio = 0;
            for first_char in first.chars() {
                let found_char = second
                    .chars()
                    .find(|second_char| second_char.eq(&first_char));
                if let Some(char) = found_char {
                    prio = char_to_prio(char);
                    break;
                }
            }
            prio
        })
        .sum()
}

pub fn process_part2(input: &str) -> i32 {
    let mut lines = input.lines();
    let mut result = 0;
    while let Some(elf_1) = lines.next() {
        let elf_2 = lines.next().unwrap();
        let elf_3 = lines.next().unwrap();
        for elf_1_char in elf_1.chars() {
            if elf_2.contains(elf_1_char) && elf_3.contains(elf_1_char) {
                result += char_to_prio(elf_1_char);
                break;
            }
        }
    }
    result
}

#[cfg(test)]
mod tests {
    use super::*;

    const INPUT: &str = "vJrwpWtwJgWrhcsFMMfFFhFp
jqHRNqRjqzjGDLGLrsFMfFZSrLrFZsSL
PmmdzqPrVvPwwTWBwg
wMqvLMZHhHMvwLHjbvcjnnSBnvTQFn
ttgJtRGJQctTZtZT
CrZsJsPPZsGzwwsLwLmpwMDw";

    #[test]
    fn part1() {
        let result = process_part1(INPUT);
        assert_eq!(result, 157);
    }

    #[test]
    fn part2() {
        let result = process_part2(INPUT);
        assert_eq!(result, 70);
    }
}
