pub fn process_part1(input: &str) -> i32 {
    let result = input
        .lines()
        .map(|line| {
            let (elf1, elf2) = line.split_once(",").unwrap();
            let elf1 = elf1
                .split_once("-")
                .map(|(start, end)| start.parse::<i32>().unwrap()..=end.parse::<i32>().unwrap())
                .unwrap();

            let elf2 = elf2
                .split_once("-")
                .map(|(start, end)| start.parse::<i32>().unwrap()..=end.parse::<i32>().unwrap())
                .unwrap();

            if (elf1.contains(elf2.start()) && elf1.contains(elf2.end()))
                || (elf2.contains(elf1.start()) && elf2.contains(elf1.end()))
            {
                return 1;
            }
            0
        })
        .sum();

    result
}

pub fn process_part2(input: &str) -> i32 {
    let result = input
        .lines()
        .map(|line| {
            let (elf1, elf2) = line.split_once(",").unwrap();
            let elf1 = elf1
                .split_once("-")
                .map(|(start, end)| start.parse::<i32>().unwrap()..=end.parse::<i32>().unwrap())
                .unwrap();

            let elf2 = elf2
                .split_once("-")
                .map(|(start, end)| start.parse::<i32>().unwrap()..=end.parse::<i32>().unwrap())
                .unwrap();

            if elf1.contains(elf2.start())
                || elf1.contains(elf2.end())
                || elf2.contains(elf1.start())
                || elf2.contains(elf1.end())
            {
                return 1;
            }
            0
        })
        .sum();

    result
}

#[cfg(test)]
mod tests {
    use super::*;

    const INPUT: &str = "2-4,6-8
2-3,4-5
5-7,7-9
2-8,3-7
6-6,4-6
2-6,4-8";

    #[test]
    fn part1() {
        let result = process_part1(INPUT);
        assert_eq!(result, 2);
    }

    #[test]
    fn part2() {
        let result = process_part2(INPUT);
        assert_eq!(result, 4);
    }
}
